<?php
/**
 * The admin-specific functionality of the plugin.
 *
 * @link https://getqurio.com/
 * @since 1.0.0
 *
 * @package Qurio
 * @subpackage Qurio/admin
 * @author AthensLive Media Solutions <hq@getqurio.com>
 */
class Qurio_Admin
{
    /**
     * The ID of this plugin.
     *
     * @since 1.0.0
     * @access private
     * @var string $plugin_name The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since 1.0.0
     * @access private
     * @var string $version The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since 1.0.0
     * @param string $plugin_name The name of this plugin.
     * @param string $version The version of this plugin.
     */
    public function __construct($plugin_name, $version)
    {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
        add_action('rest_api_init', [$this, 'qurio_register_routes']);

        // Hook for permanent deletion
        add_action('before_delete_post', [$this, 'qurio_delete_post']);

        // Hook for trashing the post
        add_action('wp_trash_post', [$this, 'qurio_delete_post']);

        // Handle create campaign button click
        add_action('wp_ajax_qurio_create_campaign', [$this, 'qurio_handle_create_campaign']);
    }

    /**
     * Register the stylesheets for the admin area.
     *
     * @since 1.0.0
     */
    public function qurio_enqueue_styles()
    {
        wp_enqueue_style(
            $this->plugin_name,
            plugin_dir_url(__FILE__) . 'css/qurio-admin.css',
            [],
            $this->version,
            'all'
        );
    }

    /**
     * Register the JavaScript for the admin area.
     *
     * @since 1.0.0
     */
    public function qurio_enqueue_scripts()
    {
        wp_enqueue_script(
            $this->plugin_name,
            plugin_dir_url(__FILE__) . 'js/qurio-admin.js',
            ['jquery'],
            $this->version,
            false
        );

        // Localize the script with AJAX URL and nonce
        wp_localize_script($this->plugin_name, 'qurioAjax', [
            'ajax_url' => admin_url('admin-ajax.php'),
            'nonce' => wp_create_nonce('qurio_create_campaign_nonce'),
            'loader_image' => plugin_dir_url(__FILE__) . 'images/qurio-spinner.gif'
        ]);

    }

    /**
     * Register API routes that will be called from Qurio SaaS
     */
    public function qurio_register_routes()
    {
        // Qurio created a new campaign
        register_rest_route('qurio/v1', '/post-campaign', [
            'methods' => 'POST',
            'callback' => [$this, 'qurio_save_campaign_data'],
            'permission_callback' => function () {
                return current_user_can( 'edit_others_posts' );
            }
        ]);
        // Qurio wants to update a WP post
        register_rest_route("qurio/v1", "/trigger-update", [
            "methods" => "POST",
            "callback" => [$this, "qurio_trigger_post_update"],
            'permission_callback' => function () {
                return current_user_can( 'edit_others_posts' );
            }
        ]);
        // Qurio sends us new appearance
        register_rest_route("qurio/v1", "/post-appearance", [
            "methods" => "POST",
            "callback" => [$this, "qurio_save_appearance_data"],
            'permission_callback' => function () {
                return current_user_can( 'edit_others_posts' );
            }
        ]);
    }

    /**
     * Get the current API key
     */
    public function qurio_get_api_key()
    {
        return esc_attr(get_option('qurio_api_key'));
    }

    /*
     * Add a settings link on the plugin page
     */
    public function qurio_add_settings_link($links)
    {
        $settings_link =
            '<a href="' .
            admin_url('admin.php?page=qurio-settings') .
            '">Settings</a>';
        array_push($links, $settings_link);
        return $links;
    }

    /**
     * Send a POST to Qurio API endpoint
     * @param $action str 'widget' add in url
     * @param $body array fields to be passed as the API body
     * @param $headers optional field
     *
     * @since 1.0.0
     *
     * @return encoded json
     */
    public function qurio_send_api_request($action, $body = [], $headers = [])
    {
        // Need API key for all API requests
        $api_key = $this->qurio_get_api_key();
        if (!$api_key) {
            return wp_json_encode([
                'status' => 'error',
                'code' => 422,
                'message' => 'Missing API key',
            ]);
        }

        $action = sanitize_text_field($action);
        $url = esc_url_raw(QURIO_API_ROOT . '/' . $action);
        $site_url = get_site_url();

        qurio_write_log("Sending a POST to {$url}");

        $headers['Content-Type'] = 'application/json';
        $headers['Authorization'] = $api_key;
        $headers['quriowpdomain'] = $site_url;
        $headers['Qurio-Plugin-Version'] = QURIO_VERSION;
        $headers['Source'] = 'qurio-wp-plugin';

        if (is_array($body) && is_array($headers)) {
            $post_fields = [
                'method' => 'POST',
                'headers' => $headers,
                'cookies' => [],
                'timeout' => 30,
                'redirection' => 10,
                'httpversion' => '1.1',
                'blocking' => true,
                'body' => empty($body) ? '' : wp_json_encode($body),
            ];

            $response = wp_remote_post($url, $post_fields);

            if (is_wp_error($response)) {
                // Handle error
                $error_message = sanitize_text_field($response->get_error_message());
                qurio_write_log("Error sending POST to {$url} with {$error_message}", 'error');
                return wp_json_encode([
                    'status' => 'error',
                    'code' => $response['response']['code'],
                    'message' => $error_message,
                ]);
            } else {
                // The API call was successful
                qurio_write_log("POST call to {$url} was successful");
                $code = isset($response['response']['code']) ? $response['response']['code'] : 200;
                return wp_json_encode([
                    'status' => 'success',
                    'code' => $code,
                    'results' => wp_remote_retrieve_body($response),
                ]);
            }
        } else {
            qurio_write_log("422 Invalid POST request {$url}", 'error');
            return wp_json_encode([
                'status' => 'error',
                'code' => 422,
                'message' => 'Invalid API request',
            ]);
        }
    }

    /**
     * Save the api key in wordpress and perform handshake if needed
     */
    public function qurio_update_api_key()
    {
        if (!isset($_SERVER['REQUEST_METHOD']) || $_SERVER['REQUEST_METHOD'] !== 'POST') {
            return;
        }

        // Check if the request is coming from the correct page
        if (!isset($_POST['page']) || $_POST['page'] !== 'qurio-settings') {
            return;
        }
        
        if (!isset($_POST['qurio_save_api_key_nonce']) || !wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['qurio_save_api_key_nonce'])), 'qurio_save_api_key_nonce_action') ) {
                // Nonce verification failed, display an error message
                echo '<div class="error notice is-dismissible"><p>Security check failed. Please try again.</p></div>';
                return;
         }

        // Get submitted API key from form submission
        $new_api_key = '';
        if (isset($_POST['qurio_api_key'])) {
            $new_api_key = sanitize_text_field(wp_unslash($_POST['qurio_api_key']));
        }

        // Disconnect from Qurio if new API key is empty
        if (empty($new_api_key)) {
            $result = $this->qurio_unbind_api();
        } else {
            // Current API key
            $current_api_key = $this->qurio_get_api_key();

            // Determine the action and prepare the log message
            if (empty($current_api_key)) {
                qurio_write_log("New API key added: {$new_api_key}");
            } elseif (!empty($current_api_key) && $current_api_key !== $new_api_key) {
                qurio_write_log("API key changed from {$current_api_key} to {$new_api_key}");
            } else {
                qurio_write_log("API key remains unchanged: {$new_api_key}");
            }

            // Perform handshake
            update_option('qurio_api_key', $new_api_key);
            $result = $this->qurio_handshake_api();

            // Send last 50 posts from WP to Qurio
            $this->qurio_send_last_posts();
        }

        register_setting('qurio_settings_group', 'qurio_api_key', [
            'type'              => 'string',
            'sanitize_callback' => 'sanitize_text_field',
        ]);
        register_setting('qurio_settings_group', 'qurio_api_status', [
            'type'              => 'string',
            'sanitize_callback' => 'sanitize_text_field',
        ]);
        
    }

    /**
     * Call Qurio Handshake API endpoint to register this WP
     */
    public function qurio_handshake_api()
    {
        $current_api_key = $this->qurio_get_api_key();
        $site_url = esc_url(get_site_url());
        $site_title = get_bloginfo('name');
        $code = get_bloginfo('language');

        $languages = [
            'en-US' => 'English',
            'en-GB' => 'English',
            'es-ES' => 'Spanish',
            'es-MX' => 'Spanish',
            'fr-FR' => 'French',
            'de-DE' => 'German',
            'it-IT' => 'Italian',
            'pt-PT' => 'Portuguese',
            'pt-BR' => 'Portuguese',
            'el-EL' => 'Greek',
        ];

        // Default to 'Unknown Language' if the code is not found
        $language = isset($languages[$code]) ? $languages[$code] : 'Unknown Language';

        $body = [
            'apiKey' => $current_api_key,
            'apiUrl' => $site_url,
            'publicationName' => $site_title,
            'publicationPlatform' => 'WordPress',
            'publicationLanguage' => $language,
        ];

        // Make the remote request
        $action = 'handshake-wordpress';
        $api_response = $this->qurio_send_api_request($action, $body);
        $api_response = json_decode($api_response, true);

        // If API returns success, respond accordingly
        if ($api_response['status'] == 'success' && $api_response['code'] == 200) {
            qurio_write_log("Handshake done with key {$current_api_key}");
        } else {
            qurio_write_log("Can't do the handshake with key {$current_api_key} with code {$api_response['code']}", "error");
        }
    }

    /**
     * Call Qurio Unbind API endpoint to disconnect this WP
     */
    public function qurio_unbind_api()
    {
        $current_api_key = $this->qurio_get_api_key();

        // Make the remote request
        $action = 'unbind-wordpress';
        $body = ['apiKey' => $current_api_key];
        $api_response = $this->qurio_send_api_request($action, $body);
        $api_response = json_decode($api_response, true);

        // If API returns success, respond accordingly
        if ($api_response['status'] == 'success' && $api_response['code'] == 200) {
            qurio_write_log("API key disconnected from Qurio - {$current_api_key}");
        } else {
            qurio_write_log("Can't unbind API key {$current_api_key} with code {$api_response['code']}", "error");
        }
    }

    /**
     * Send last 50 posts to Qurio SaaS
     */
    public function qurio_send_last_posts()
    {
        $action = 'wp/posts';
        $post_data = $this->qurio_get_post_data();
        $this->qurio_send_api_request($action, $post_data);
    }

    /**
     * Prepare post array so it can be sent to Qurio
     */
    public function qurio_get_post_data($post_id = null)
    {
        // Prepare the base query arguments
        $args = [
            'post_status' => [
                'publish',
                'draft',
                'pending',
                'future',
                'private',
            ],
            'post_type' => 'post',
        ];

        // If a specific post ID is passed, modify the query to get only that post
        if ($post_id) {
            $args['p'] = $post_id; // 'p' parameter is used to query by post ID
            qurio_write_log("Fetching one post with ID {$post_id} from WP");
        } else {
            // If no post ID is passed, fetch multiple posts with the default limit
            $args['posts_per_page'] = 50;
            qurio_write_log("Fetching last 50 posts from WP");
        }

        // Run the query
        $posts_query = new WP_Query($args);
        $posts_details = [];
        if ($posts_query->have_posts()) {
            while ($posts_query->have_posts()) {
                $posts_query->the_post();
                global $post;
                // Create a new array to store the post data
                $post_arr = [];
                $post_arr['platform'] = 'WordPress';
                $post_arr['external_id'] = get_the_ID();
                $post_arr['post_url'] = get_permalink();
                $post_arr['status'] = $post->post_status;
                $post_arr['title'] = get_the_title();
                $post_arr['content'] = $post->post_content;

                // New date formats
                $post_arr['published_date'] = gmdate('Y-m-d H:i:s', strtotime($post->post_date_gmt));
                $post_arr['created_date'] = gmdate('Y-m-d H:i:s', strtotime($post->post_date_gmt));
                $post_arr['modified_date'] = gmdate('Y-m-d H:i:s', strtotime($post->post_modified_gmt));

                // Get the full URL of the post thumbnail
                $post_image = get_the_post_thumbnail_url(get_the_ID(), 'full');
                
                // If no featured image, grab the first image in the post content
                if (!$post_image) {
                    $post_image = $this->qurio_extract_image_src($post->post_content);
                }

                $post_arr['image_url'] = $post_image;

                // Get author details
                $author = get_user_by('id', $post->post_author);
                if ($author) {
                    $post_arr['author_name'] = $author->display_name;
                    $post_arr['author_id'] = $author->ID;
                } else {
                    $post_arr['author_name'] = '';
                    $post_arr['author_id'] = '';
                }

                // Add the post details to the array
                $posts_details[] = $post_arr;
            }
        }
        wp_reset_postdata();
        return $posts_details;
    }

    /**
     * Extract the first image src from the post content
     * 
     * @param string $content The post content  
     * @return string|null The first image src or null if no image is found 
     */
    private function qurio_extract_image_src($content)
    {
        if (empty($content)) {
            return null;
        }

        $dom = new DOMDocument();
        @$dom->loadHTML($content);
        $images = $dom->getElementsByTagName('img');

        if ($images->length > 0) {
            return $images->item(0)->getAttribute('src');
        }
        return null;
    }

    /**
     * Update Qurio when a WP post has been changed
     */
    public function qurio_send_post_data($post_id, $post, $update)
    {
        // Exclude other post types, check for default posts only
        if (get_post_type($post_id) !== 'post') {
            return;
        }

        // If this is a deleted or new post, don't run API.
        if ($post->post_status == 'trash' || $post->post_status == 'auto-draft') {
            return;
        }

        // Make the remote request
        $post_data = $this->qurio_get_post_data($post_id);
        $api_response = $this->qurio_send_api_request('wp/posts', $post_data);
        $api_response = json_decode($api_response, true);

        // If API returns success, respond accordingly
        if ($api_response['status'] == 'success' && $api_response['code'] == 200) {
            qurio_write_log("Post ID {$post_id} data sent to Qurio successfully.");
        } else {
            qurio_write_log("Failed to send post {$post_id} data via API", "error");
        }
    }

    /**
     * Add a Qurio column to the /posts table
     */
    public function qurio_post_column_connect($columns)
    {
        $columns['qurio_status'] = 'Qurio';
        return $columns;
    }

    /**
     * Render correct link in the Qurio column
     */
    public function qurio_campaign_column($column, $post_id)
    {
        // Only proceed if this is the Qurio status column
        if ($column !== 'qurio_status') {
            return;
        }

        // Check if API key is set
        $current_api_key = $this->qurio_get_api_key();
        if (empty($current_api_key)) {
            return;
        }

        // Get the campaign ID associated with this post
        $campaign_id = get_post_meta($post_id, 'qurio_campaign_id', true);

        if ($campaign_id) {
            // If a campaign exists, display a "View Campaign" link
            $link_url = QURIO_APP_URL . '/campaign/view/' . $campaign_id;
            echo esc_url($this->qurio_render_view_campaign_link($link_url));
        } else {
            // If no campaign exists, display a "Create Campaign" button
            $link_url = QURIO_APP_URL . '/campaign/new/step2?post_id=' . $post_id;
            echo  $this->qurio_render_create_campaign_button($post_id, esc_url($link_url));
        }
    }

    private function qurio_render_view_campaign_link($url)
    {
        
        return sprintf(
            // translators: %s is the URL for the campaign link.
            __('<a href="%s" target="_blank">View Campaign</a>', 'qurio-client-tool'),
            esc_url($url)
        );
    }

    private function qurio_render_create_campaign_button($post_id, $redirect_url)
    {   
        return sprintf(
              // translators: %s is the URL for redirection and %d is for post id.
            __('<a href="javascript:void(0);" class="ctm_qurio_btn" data-post-id="%1$d" data-redirect-url="%2$s">Create Campaign</a>', 'qurio-client-tool'),
            esc_attr($post_id),
            esc_url($redirect_url)
        );
    }

    /**
     * AJAX function for the Qurio column
     */
    public function qurio_handle_create_campaign()
    {
        // Verify nonce for security
        check_ajax_referer('qurio_create_campaign_nonce', 'nonce');

        if (!isset($_POST['post_id'])) {
            qurio_write_log('Invalid request to handle create campaign', 'error');
            wp_send_json_error(['message' => 'Invalid request.']);
        }

        $post_id = intval($_POST['post_id']);

        // Sanitize and validate the post ID
        if (!get_post($post_id)) {
            qurio_write_log("Post id {$post_id} not found when handling create campaign", "error");
            wp_send_json_error(['message' => 'Post not found.']);
        }

        $action = 'wp/posts';
        $post_data = $this->qurio_get_post_data($post_id);

        if (!$post_data) {
            qurio_write_log("Failed to get post {$post_id} data when handling create campaign", "error");
            wp_send_json_error(['message' => 'Failed to get post data.']);
        }

        $api_response = $this->qurio_send_api_request($action, $post_data);
        $api_response = json_decode($api_response, true);

        // If API returns success, respond accordingly
        if ($api_response['status'] == 'success' && $api_response['code'] == 200) {
            qurio_write_log("Sent post {$post_id} data to Qurio when handling create campaign");
            wp_send_json_success(['code' => 200]);
        } else {
            qurio_write_log('Error creating campaign when handling create campaign', 'error');
            wp_send_json_error(['message' => 'Error creating campaign.']);
        }
    }

    /**
     * Receive campaign data from Qurio and save it in WP
     */
    public function qurio_save_campaign_data($data)
    {
        $post_id = absint($data['post_id']);
        if ($post_id > 0) {

            $campaign_id = sanitize_text_field($data['campaign_id']);
            if (empty($campaign_id)) {
                // Delete campaign info
                update_post_meta($post_id, 'qurio_campaign_id', '');
                update_post_meta($post_id, 'qurio_connect_campaign_style', '');

                $result_message = "For Post ID {$post_id}, Campaign was deleted.";
            } else {
                // Update post metadata with campaign info
                $campaign_display = sanitize_text_field($data['style']);

                update_post_meta($post_id, 'qurio_campaign_id', $campaign_id);
                update_post_meta($post_id, 'qurio_connect_campaign_style', $campaign_display);

                $result_message = "For Post ID {$post_id}, Saved Campaign: {$campaign_id}, Style: {$campaign_display}";
            }

            // Update post date to trigger content refresh
            wp_update_post(['ID' => $post_id], true);
        } else {
            $result_message = "Can't update campaign data - no Post with ID {$post_id} found.";
        }

        qurio_write_log($result_message);
        $api_data = [
            'post_id' => $post_id,
            'campaign_id' => $campaign_id,
            'msg' => $result_message,
            'dataget' => wp_json_encode($data),
        ];
        return new WP_REST_Response($api_data);
    }

    /**
     * Qurio SaaS wants to trigger a post update
     */
    public function qurio_trigger_post_update(WP_REST_Request $request)
    {
        // Get POST data
        $post_id = isset($request['post_id']) ? intval($request['post_id']) : 0;
        if (empty($post_id)) {
            qurio_write_log('Post ID is missing when triggering post update', 'error');
            return new WP_Error('missing_post_id', 'Post ID is missing', ['status' => 400]);
        }

        // Update the post (no content change)
        $result = wp_update_post(["ID" => $post_id], true);
        if (is_wp_error($result)) {
            qurio_write_log("Failed to trigget post {$post_id} update", "error");
            return $result;
        }

        qurio_write_log("Post {$post_id} update triggered successfully");
        return new WP_REST_Response(['message' => 'Post update triggered successfully'], 200);
    }

    /**
     * Save appearance data received from Qurio SaaS
     *
     * @param array $data The appearance data
     * @return WP_REST_Response
     */
    public function qurio_save_appearance_data($data)
    {
        // Sanitize and extract appearance data
        $overlay_color = sanitize_text_field($data['overlayColor']);
        $overlay_opacity = sanitize_text_field($data['overlayOpacity']);
        $popup_delay = sanitize_text_field($data['popupDelay']);
        $inline_position = sanitize_text_field($data['inlinePosition']);

        // Update WordPress options with the new appearance data
        update_option('qurio_appearance_overlay_color', $overlay_color);
        update_option('qurio_appearance_overlay_opacity', $overlay_opacity);
        update_option('qurio_appearance_popup_delay', $popup_delay);
        update_option('qurio_appearance_inline_position', $inline_position);

        // Log the saved appearance data
        $log_message = sprintf(
            "Appearance data saved: Overlay Color: %s, Overlay Opacity: %s, Popup Delay: %s, Inline Position: %s",
            $overlay_color,
            $overlay_opacity,
            $popup_delay,
            $inline_position
        );
        qurio_write_log($log_message);

        // Return a success response
        return new WP_REST_Response(
            ['message' => 'Appearance data saved successfully'],
            200
        );
    }

    /**
     * Get campaign stats from Qurio SaaS
     */
    public function qurio_get_campaign_stats()
    {
        $response = $this->qurio_send_api_request('wp/campaigns-stats');
        $response_data = json_decode($response, true);

        // Check for errors in the response
        if (
            (is_array($response_data) && !empty($response_data)) ||
            json_last_error() !== JSON_ERROR_NONE
        ) {
            if ($api_response['status'] == 'success' && $api_response['code'] == 200) {
                $results = sanitize_textarea_field($response_data['results']);
                if (!$results) {
                    return;
                }
                $decoded_results = json_decode($results, true);
                $saved = update_option('qurio_overview_campaign_data', maybe_serialize($decoded_results));
            }
        }
    }

    /**
     * Post has been deleted in WP, also delete its info in Qurio SaaS
     */
    public function qurio_delete_post($post_id)
    {
        // Check if the post ID is valid and sanitize it
        $post_id = absint($post_id);
        if (!$post_id || get_post_type($post_id) !== 'post') {
            return;
        }

        // Make the remote request
        $api_response = $this->qurio_send_api_request('wp/deletepost', ['ID' => $post_id]);
        $api_response = json_decode($api_response, true);

        // If API returns success, respond accordingly
        if ($api_response['status'] == 'success' && $api_response['code'] == 200) {
            qurio_write_log("Post ID {$post_id} deleted/trashed via API successfully.");
        } else {
            qurio_write_log("Failed to delete post {$post_id} via API with code {$api_response['code']}", "error");
        }
    }

    /**
     * Create plugin settings admin menu
     */
    public function qurio_create_menu()
    {
        // Get the plugin directory URL to use for the icon image
        $icon_url = '<svg width="25" height="25" viewBox="0 0 80 80" xmlns="http://www.w3.org/2000/svg">
  <style>
    .default { fill: #dcdcdc; } /* WP light grey */
    .hover:hover { fill: #007cba; } /* WP blue on hover */
    .active { fill: #ffffff; } /* WP white on active */
    .toplevel_page_qurio-settings img {
        padding-top: 5px !important;
    }
  </style>
  <g class="default hover active">
    <path fill-rule="evenodd" clip-rule="evenodd" d="M39.9764 54.4C52.8338 54.4 63.2567 44.3712 63.2567 32C63.2567 19.6288 52.8338 9.6 39.9764 9.6C27.119 9.6 16.696 19.6288 16.696 32C16.696 44.3712 27.119 54.4 39.9764 54.4ZM39.9764 64C58.344 64 73.234 49.6731 73.234 32C73.234 14.3269 58.344 0 39.9764 0C21.6087 0 6.71875 14.3269 6.71875 32C6.71875 49.6731 21.6087 64 39.9764 64ZM36.1403 44.0155C36.0681 41.8075 37.8701 39.9612 40.1648 39.8918C41.5204 39.8508 42.845 39.4914 44.0231 38.8448C45.2012 38.1983 46.197 37.2842 46.9237 36.1824C47.6505 35.0805 48.086 33.8244 48.1924 32.5234C48.2988 31.2225 48.0729 29.9162 47.5343 28.7186C46.9956 27.5209 46.1606 26.4682 45.1022 25.6523C44.0437 24.8364 42.7939 24.282 41.4617 24.0374C40.1294 23.7929 38.7552 23.8657 37.4589 24.2494C36.1626 24.6331 34.9836 25.3162 34.0245 26.2389C32.401 27.801 29.7688 27.801 28.1453 26.2389C26.5219 24.6768 26.5219 22.1441 28.1453 20.5821C30.0634 18.7365 32.4215 17.3705 35.0142 16.6031C37.6068 15.8356 40.3552 15.6902 43.0196 16.1792C45.6841 16.6682 48.1838 17.777 50.3007 19.4089C52.4177 21.0407 54.0876 23.1461 55.1649 25.5414C56.2421 27.9367 56.6941 30.5492 56.4812 33.1511C56.2684 35.753 55.3973 38.2653 53.9438 40.469C52.4904 42.6727 50.4987 44.5008 48.1425 45.7939C45.7863 47.087 43.1372 47.8059 40.426 47.8879C38.1311 47.9572 36.2124 46.2235 36.1403 44.0155ZM39.9765 80C43.65 80 46.628 77.1348 46.628 73.6C46.628 70.0652 43.65 67.2002 39.9765 67.2002C36.3029 67.2002 33.3249 70.0652 33.3249 73.6C33.3249 77.1348 36.3029 80 39.9765 80Z"/>
  </g>
</svg>';
        $icon_url = 'data:image/svg+xml;base64,' . base64_encode($icon_url);

        add_menu_page(
            'Qurio Page',
            'Qurio',
            'manage_options',
            'qurio-settings',
            [$this, 'qurio_settings_page'],
            $icon_url
        );
        add_submenu_page(
            'qurio-settings',
            'Settings Page',
            'Settings',
            'manage_options',
            'qurio-settings'
        );
        add_submenu_page(
            'qurio-settings',
            'Overview Page',
            'Overview',
            'manage_options',
            'qurio-overview',
            [$this, 'qurio_overview_page']
        );

        add_action('admin_init', [$this, 'qurio_update_api_key']);
    }

    /**
     * Render Qurio settings page
     */
    public function qurio_settings_page()
    {
        $qurio_api_key = $this->qurio_get_api_key();
        // Check if the form is submitted
        if (isset($_SERVER['REQUEST_METHOD']) && $_SERVER['REQUEST_METHOD'] === 'POST') {
            // Verify the nonce
            if (
                !isset($_POST['qurio_save_api_key_nonce']) ||
                !wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['qurio_save_api_key_nonce'])), 'qurio_save_api_key_nonce_action')
            ) {
                // Nonce verification failed, display an error message
                echo '<div class="error notice is-dismissible"><p>Security check failed. Please try again.</p></div>';
                return;
            }
        }

        // Process form data here after nonce verification
        $qurio_api_status = esc_attr(get_option('qurio_api_status'));

        // Display settings errors registered
        if ($settings_errors = get_transient('settings_errors')) {
            foreach ($settings_errors as $error) {
                echo '<div class="' .
                    esc_attr($error["type"]) .
                    ' notice is-dismissible"><p>' .
                    esc_html($error["message"]) .
                    "</p></div>";
            }

            delete_transient('settings_errors'); // Clear the transient after displaying the errors
        }
        ?>
        <div class="wrap ctm_main_wrap">
            <h1>Qurio Settings</h1>
            <p>To get the Qurio API key, go to your Qurio account, click on the profile icon in the top right, and copy the API key from there.</p>
            <form method="post" action="<?php echo esc_url(
                admin_url("options.php")
            ); ?>">
                <?php settings_fields("qurio_settings_group"); ?>
                <?php do_settings_sections("qurio_settings_group"); ?>

                <!-- Add the nonce field -->
                <?php wp_nonce_field(
                    "qurio_save_api_key_nonce_action",
                    "qurio_save_api_key_nonce"
                ); ?>

                <input type="hidden" name="page" value="qurio-settings" />
                <table class="form-table ctm_form_table">
                    <tr valign="top">
                        <th scope="row"><label for="qurio_api_key">Qurio API key</label></th>
                        <td>
                            <input type="text" name="qurio_api_key" id="qurio_api_key" value="<?php echo esc_attr(
                                $qurio_api_key
                            ); ?>" />
                        </td>
                    </tr>
                </table>
                <?php submit_button("Save Changes"); ?>
            </form>
            <hr>
            <div class="qurio_widget_instructions">
                <h2>Instructions:</h2>
                <div class="">
                    <p>Qurio Engagements Widgets has 3 display options for your WordPress website.</p>
                    <ol>
                        <li><strong>Inline. </strong>The inline widget will natively appear inside the article after the 2nd paragraph. You can set a custom widget position with the shortcode shown below. Just add it to the post where you would like to see the widget.<br>
                            <input type="text" readonly disabled id="qurio_shortcode_field" value="[qurio_embed_campaign]">
                            <button type="button" id="qurio_shortcode_copy">Copy Shortcode</button></li>
                        <li><strong>Popup. </strong>Popup widgets show automatically on page load after a certain delay. To configure the delay, go to the Appearances page in your Qurio account.</li>
                        <li><strong>Standalone. </strong>Every campaign has a unique Standalone URL. You can link it directly with your posts, emails, or social media.</li>
                    </ol>
                </div>
            </div>

            <div class="qurio_plugin_version">
                <b>Plugin version :</b> <?php echo esc_html(QURIO_VERSION); ?>
            </div>
        </div>
        <?php
    }

    /**
     * Render Qurio overview page
     */
    public function qurio_overview_page()
    {
        $this->qurio_get_campaign_stats();
        $campaigns = maybe_unserialize(get_option('qurio_overview_campaign_data'));
        if (!is_array($campaigns)) {
            $campaigns = [];
        }
        // Initialize a counter for the "published" status
        $active_campaign = 0;
        $total_campaign = 0;
        $inactive_campaign = 0;
        $responses_last_month = 0;
        if (is_array($campaigns) && !empty($campaigns)) {
            $total_campaign = count($campaigns); // Iterate through each campaign and increment the counter if the status is "published"
            foreach ($campaigns as $campaign) {
                if ($campaign['status'] === 'published') {
                    $active_campaign++;
                    $responses_last_month += absint(
                        $campaign['formResponseCount']
                    );
                }
            }
            $inactive_campaign = $total_campaign - $active_campaign;
        }
        
        echo wp_kses_post(
            
            sprintf(
                  // translators: quick overview.
                __(
                    '<div class="wrap ctm_main_wrap ctm_overview_main"><h1>Quick Overview</h1>',
                    "qurio-client-tool"
                )
            )
        );
        
        echo wp_kses_post(
           
            sprintf(
                 // translators: comment
                __(
                    "Here's a quick overview of how your campaigns have been doing. Log in to the app for a more detailed breakdown.",
                    "qurio-client-tool"
                )
            )
        );
        
        echo wp_kses_post(
           
            sprintf(
                 // translators: comment
                __(
                    ' <div class="ctm_form_table ctm_form_overview"><div class="ctm_box_model"><div class="ctm_box_type"><h4>Active Campaigns</h4>',
                    "qurio-client-tool"
                )
            )
        );
        
        echo wp_kses_post(
            
            sprintf(
            // translators: active_campaign
                __(
                    ' <h5>%d</h5></div>',
                    "qurio-client-tool"
                ),
                esc_attr($active_campaign)
            )
        );
        
        echo wp_kses_post(
            
            sprintf(
              // translators: comment
                __(
                    ' <div class="ctm_box_type border_box_left"><h4>Reponses gathered during the last 30 days</h4>',
                    "qurio-client-tool"
                )
            )
        );
        
        echo wp_kses_post(
            
            sprintf(
              // translators: %d is response last month
                __(
                    ' <h5>%d</h5></div></div>',
                    "qurio-client-tool"
                ),
                esc_attr($responses_last_month)
            )
        );
        
        echo wp_kses_post(
            
            sprintf(
              // translators: %s is app url
                __(
                    ' <div class="ctm_over_view_button"><a href="%s" target="_blank" class="button-primary">Go to Qurio</a></div></div></div>',
                    "qurio-client-tool"
                ),
                esc_url(QURIO_APP_URL)
            )
        );
    }
}
