<?php
/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://getqurio.com/
 * @since      1.0.0
 *
 * @package    Qurio
 * @subpackage Qurio/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Qurio
 * @subpackage Qurio/includes
 * @author     AthensLive Media Solutions <hq@getqurio.com>
 */
class Qurio
{
    /**
     * The loader that's responsible for maintaining and registering all hooks that power
     * the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      Qurio_Loader    $loader    Maintains and registers all hooks for the plugin.
     */
    protected $loader;

    /**
     * The unique identifier of this plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $plugin_name    The string used to uniquely identify this plugin.
     */
    protected $plugin_name;

    /**
     * The current version of the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $version    The current version of the plugin.
     */
    protected $version;

    /**
     * Define the core functionality of the plugin.
     *
     * Set the plugin name and the plugin version that can be used throughout the plugin.
     * Load the dependencies, define the locale, and set the hooks for the admin area and
     * the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function __construct()
    {
        if (defined('QURIO_VERSION')) {
            $this->version = QURIO_VERSION;
        } else {
            $this->version = '1.0.0';
        }
        $this->plugin_name = 'qurio';

        $this->qurio_load_dependencies();
        $this->qurio_set_locale();
        $this->qurio_define_admin_hooks();
        $this->qurio_define_public_hooks();
    }

    /**
     * Load the required dependencies for this plugin.
     *
     * Include the following files that make up the plugin:
     *
     * - Qurio_Loader. Orchestrates the hooks of the plugin.
     * - Qurio_i18n. Defines internationalization functionality.
     * - Qurio_Admin. Defines all hooks for the admin area.
     * - Qurio_Public. Defines all hooks for the public side of the site.
     *
     * Create an instance of the loader which will be used to register the hooks
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function qurio_load_dependencies()
    {
        /**
         * The class responsible for orchestrating the actions and filters of the
         * core plugin.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-qurio-loader.php';

        /**
         * The class responsible for defining internationalization functionality
         * of the plugin.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-qurio-i18n.php';

        /**
         * The class responsible for defining all actions that occur in the admin area.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'admin/class-qurio-admin.php';

        /**
         * The class responsible for defining all actions that occur in the public-facing
         * side of the site.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'public/class-qurio-public.php';

        $this->loader = new Qurio_Loader();
    }

    /**
     * Define the locale for this plugin for internationalization.
     *
     * Uses the Qurio_i18n class in order to set the domain and to register the hook
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function qurio_set_locale()
    {
        $plugin_i18n = new Qurio_i18n();

        $this->loader->add_action('plugins_loaded', $plugin_i18n, 'qurio_load_plugin_textdomain');
    }

    /**
     * Register all of the hooks related to the admin area functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function qurio_define_admin_hooks()
    {
        $plugin_admin = new Qurio_Admin(
            $this->qurio_plugin_name(),
            $this->qurio_version()
        );

        $this->loader->add_action(
            'admin_enqueue_scripts',
            $plugin_admin,
            'qurio_enqueue_styles'
        );
        $this->loader->add_action(
            'admin_enqueue_scripts',
            $plugin_admin,
            'qurio_enqueue_scripts'
        );

        $this->loader->add_action(
            'admin_menu',
            $plugin_admin,
            'qurio_create_menu'
        );
        $this->loader->add_filter(
            'manage_posts_columns',
            $plugin_admin,
            'qurio_post_column_connect'
        );
        $this->loader->add_action(
            'manage_posts_custom_column',
            $plugin_admin,
            'qurio_campaign_column',
            10,
            2
        );
        $this->loader->add_action(
            'save_post',
            $plugin_admin,
            'qurio_send_post_data',
            10,
            3
        );
        $this->loader->add_filter(
            'plugin_action_links_qurio/qurio.php',
            $plugin_admin,
            'qurio_add_settings_link'
        );
    }

    /**
     * Register all of the hooks related to the public-facing functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function qurio_define_public_hooks()
    {
        $plugin_public = new Qurio_Public(
            $this->qurio_plugin_name(),
            $this->qurio_version()
        );

        $this->loader->add_action(
            'wp_enqueue_scripts',
            $plugin_public,
            'qurio_enqueue_styles'
        );
        $this->loader->add_action(
            'wp_enqueue_scripts',
            $plugin_public,
            'qurio_enqueue_scripts'
        );
    }

    /**
     * Run the loader to execute all of the hooks with WordPress.
     *
     * @since    1.0.0
     */
    public function run()
    {
        $this->loader->run();
    }

    /**
     * The name of the plugin used to uniquely identify it within the context of
     * WordPress and to define internationalization functionality.
     *
     * @since     1.0.0
     * @return    string    The name of the plugin.
     */
    public function qurio_plugin_name()
    {
        return $this->plugin_name;
    }

    /**
     * The reference to the class that orchestrates the hooks with the plugin.
     *
     * @since     1.0.0
     * @return    Qurio_Loader    Orchestrates the hooks of the plugin.
     */
    public function qurio_loader()
    {
        return $this->loader;
    }

    /**
     * Retrieve the version number of the plugin.
     *
     * @since     1.0.0
     * @return    string    The version number of the plugin.
     */
    public function qurio_version()
    {
        return $this->version;
    }
}
