<?php
/**
 * The public-facing functionality of the plugin.
 *
 * @link https://getqurio.com/
 * @since 1.0.0
 *
 * @package Qurio
 * @subpackage Qurio/public
 */
/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package Qurio
 * @subpackage Qurio/public
 * @author AthensLive Media Solutions <hq@getqurio.com>
 */
class Qurio_Public
{
    /**
     * The ID of this plugin.
     *
     * @since 1.0.0
     * @access private
     * @var string $plugin_name The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since 1.0.0
     * @access private
     * @var string $version The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since 1.0.0
     * @param string $plugin_name The name of the plugin.
     * @param string $version The version of this plugin.
     */
    public function __construct($plugin_name, $version)
    {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
        add_shortcode('qurio_embed_campaign', [$this, 'qurio_insert_shortcode_widget']);
        add_action('wp_footer', [$this, 'qurio_insert_popup_widget']);
        add_filter('the_content', [$this, 'qurio_insert_inline_widget']);
    }

    /**
     * Enqueues the public-facing stylesheet.
     */
    public function qurio_enqueue_styles()
    {
        wp_enqueue_style(
            $this->plugin_name,
            plugin_dir_url(__FILE__) . 'css/qurio-public.css',
            [],
            $this->version,
            'all'
        );
    }

    /**
     * Enqueues the public-facing JavaScript.
     */
    public function qurio_enqueue_scripts()
    {
        wp_enqueue_script(
            $this->plugin_name,
            plugin_dir_url(__FILE__) . 'js/qurio-public.js',
            ['jquery'],
            $this->version,
            false
        );
    }

    /**
     * Retrieves the Qurio API key from WordPress options.
     * @return string The sanitized API key.
     */
    public function qurio_get_api_key()
    {
        return sanitize_text_field(get_option('qurio_api_key', ''));
    }

    /**
     * Gets the current post ID.
     * @return int The current post ID or 0 if not found.
     */
    private function qurio_get_current_post_id()
    {
        global $post;
        
        if (is_singular()) {
            return get_the_ID();
        }
        
        return $post instanceof WP_Post ? $post->ID : 0;
    }

    /**
     * Fetches campaign data from the Qurio API.
     * @param string $campaign_id The ID of the campaign.
     * @return array|null The campaign data or null if unsuccessful.
     */
    private function qurio_get_campaign_data($campaign_id)
    {
        $qurio_api_key = $this->qurio_get_api_key();
        if (!$qurio_api_key) {
            return null;
        }

        // Make a GET request to the Qurio API
        $response = wp_remote_get('https://app.getqurio.com/api/campaign?campaignId=' . $campaign_id, [
            'headers' => [
                'Source' => 'wp',
                'Authorization' => $qurio_api_key,
            ],
        ]);

        // Check if the request was successful
        if (is_wp_error($response)) {
            //error_log('Qurio API Error: ' . $response->get_error_message());
            return null;
        }   
        
        // Decode the JSON response and return as an associative array
        $result = wp_remote_retrieve_body($response);
        return json_decode($result, true);
    }

    /**
     * Validates if the current campaign is valid for the given post and style.
     * @param int $post_id The post ID.
     * @param string $expected_style The expected campaign style.
     * @return bool True if the campaign is valid, false otherwise.
     */
    private function qurio_is_valid_campaign($post_id, $expected_style)
    {
        // Only proceed for singular post pages
        if (!is_singular('post')) {
            return false;
        }

        // Only proceed for post type 'post'
        if (get_post_type($post_id) !== 'post') {
            return false;
        }

        // Retrieve campaign details from post meta
        $campaign_id = get_post_meta($post_id, 'qurio_campaign_id', true);
        $campaign_style = get_post_meta($post_id, 'qurio_connect_campaign_style', true) ?: 'inline';

        // Ensure campaign exists and style matches expected
        if (!$campaign_id || $campaign_style !== $expected_style) {
            return false;
        }

        // Double check campaign data with Qurio API
        $campaign_data = $this->qurio_get_campaign_data($campaign_id);
        if (!$campaign_data) {
            return false;
        }

        // Verify campaign details match current post and are valid
        return (
            isset($campaign_data['postId'], $campaign_data['displayPosition'], $campaign_data['status']) &&
            $campaign_data['postId'] == $post_id &&
            $campaign_data['displayPosition'] === $expected_style &&
            $campaign_data['status'] === 'published'
        );

    }

    /**
     * Handles the shortcode for embedding a Qurio campaign.
     * @param array $atts Shortcode attributes (unused).
     * @return string The HTML for the inline widget or an empty string.
     */
    public function qurio_insert_shortcode_widget($atts)
    {
        $post_id = $this->qurio_get_current_post_id();

        if (!$this->qurio_is_valid_campaign($post_id, 'inline')) {
            return '';
        }

        $campaign_id = get_post_meta($post_id, 'qurio_campaign_id', true);
        return $this->qurio_generate_inline_html($campaign_id, 'shortcode');
    }

    /**
     * Inserts the inline widget into the post content.
     * @param string $content The post content.
     * @return string The modified post content with the inline widget.
     */
    public function qurio_insert_inline_widget($content)
    {
        global $post;
        $post_id = $this->qurio_get_current_post_id();

        // Don't display if the shortcode is already present
        if (has_shortcode($post->post_content, 'qurio_embed_campaign')) {
            return $content;
        }

        if (!$this->qurio_is_valid_campaign($post_id, 'inline')) {
            return $content;
        }

        $campaign_id = get_post_meta($post_id, 'qurio_campaign_id', true);
        $campaign_form = $this->qurio_generate_inline_html($campaign_id);
        if (!$campaign_form) {
            return $content;
        }

        // Check if there are any paragraphs in the content
        if (strpos($content, '</p>') !== false) {
            // Insert form after the Nth paragraph, default is 2
            $inline_position = intval(get_option('qurio_appearance_inline_position', 2));
            $parts = explode('</p>', $content, $inline_position + 1);
            $modified_content = implode('</p>', array_slice($parts, 0, $inline_position));
            $modified_content .= '</p>' . $campaign_form;
            $modified_content .= implode('</p>', array_slice($parts, $inline_position));
            return $modified_content;
        } else {
            // If no paragraphs, append the form to the end of the content
            return $content . $campaign_form;
        }
    }

    /**
     * Generates the HTML for the inline widget.
     * @param string $campaign_id The campaign ID.
     * @param string $display The display style of the widget.
     * @return string The HTML for the inline widget.
     */
    private function qurio_generate_inline_html($campaign_id, $display = 'inline')
    {
        // Generate iframe source URL
        $iframe_src = esc_url(QURIO_APP_URL . '/preview-wp?campaignId=' . $campaign_id);

        // Construct output HTML
        return sprintf(
            '<div class="qurio_container qurio_widget_%1$s">
                <div class="qurio_widget" data-campaign-id="%2$s">
                    <iframe src="%3$s" frameborder="0" style="width: 100%%; max-width: 100%%;"></iframe>
                </div>
            </div>',
            $display,
            $campaign_id,
            $iframe_src
        );
    }

    /**
     * Inserts the popup widget into the footer.
     */
    public function qurio_insert_popup_widget()
    {
        $post_id = $this->qurio_get_current_post_id();

        if (!$this->qurio_is_valid_campaign($post_id, 'popup')) {
            return;
        }

        $campaign_id = get_post_meta($post_id, 'qurio_campaign_id', true);
        
        // Enqueue popup script
        $this->qurio_enqueue_popup_script($post_id);

        // Generate popup HTML
        $popup_html = $this->qurio_generate_popup_html($post_id, $campaign_id);

        // Output the popup HTML
        echo esc_html($popup_html);
    }

    /**
     * Generates the HTML for the popup widget.
     * @param int $post_id The post ID.
     * @param string $campaign_id The campaign ID.
     * @return string The HTML for the popup widget.
     */
    private function qurio_generate_popup_html($post_id, $campaign_id)
    {
        $style_css = $this->qurio_get_popup_widget_style($post_id);
        $iframe_src = esc_url(QURIO_APP_URL . '/preview-wp?campaignId=' . $campaign_id);

        return sprintf(
            '<div class="qurio_container qurio_widget_popup qurio_popup_overlay" style="display:none; %1$s">
                <div class="qurio_popup">
                    <a class="close" href="javascript:void(0)">&times;</a>
                    <div class="qurio_widget" data-campaign-id="%2$s">
                        <iframe src="%3$s" frameborder="0"></iframe>
                    </div>
                </div>
            </div>',
            $style_css,
            $campaign_id,
            $iframe_src
        );
    }

    /**
     * Generates the CSS style for the popup widget overlay.
     * @param int $post_id The post ID.
     * @return string The CSS style string.
     */
    private function qurio_get_popup_widget_style($post_id)
    {
        $overlay_color = get_option('qurio_appearance_overlay_color', '#000000');
        $overlay_opacity = get_option('qurio_appearance_overlay_opacity', 50);  
        
        list($r, $g, $b) = sscanf($overlay_color, '#%02x%02x%02x');
        $style_css = sprintf('background-color: rgba(%d, %d, %d, %.2f);', $r, $g, $b, $overlay_opacity / 100);

        return $style_css;
    }

    /**
     * Enqueues the JavaScript for the popup widget.
     * @param int $post_id The post ID.
     */
    private function qurio_enqueue_popup_script($post_id)
    {
        $popup_delay = intval(get_option('qurio_appearance_popup_delay', 3));
        
        wp_register_script('qurio-popup-script', '', '', '1.0.0', array('in_footer'  => 'true'));
        wp_enqueue_script('qurio-popup-script');
        
        $script = sprintf(
                'document.addEventListener("DOMContentLoaded", function() {
                    setTimeout(function() {
                        var overlay = document.querySelector(".qurio_popup_overlay");
                        if (overlay) {
                            overlay.style.display = "block";
                        }
                    }, %d);
                });',
                $popup_delay * 1000
        );
        wp_add_inline_script(
            'qurio-popup-script',
            $script,
            'after'
        );
    }
}